Package: ModLR
Type: Package
Title: Information-Theoretic Approach for Moderation Analysis
Version: 0.1.29
Authors@R: c(
    person(
      "Ahmad", "Daryanto",
      email = "ahdar_2000@yahoo.com",
      role = c("aut", "cre")
    )
  )
Description: Provides a robust implementation of information-theoretic moderation analysis
    using multi-model inference based on Akaike's Information Criterion (AIC) and its small-sample corrected form (Corrected AIC).
    The package enables researchers to compare competing model specifications
    and helps distinguish true interaction effects from nonlinear relationships
    that may produce spurious moderation. The methods build on Daryanto (2019)
    <doi:10.1016/j.jbusres.2019.06.012>.
License: MIT + file LICENSE
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
URL: https://github.com/ahdar1/ModLR
Imports: stats, ggplot2, broom, lmtest, sandwich, rlang
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-24 15:44:51 UTC; antob
Author: Ahmad Daryanto [aut, cre]
Maintainer: Ahmad Daryanto <ahdar_2000@yahoo.com>
Repository: CRAN
Date/Publication: 2026-05-29 10:10:08 UTC
Built: R 4.6.0; ; 2026-05-29 11:15:57 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,modlr)
export(compare_models)
export(johnson_neyman)
export(moderated_regression)
export(plot_moderation)
export(simple_slopes)
importFrom(broom,tidy)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme)
importFrom(lmtest,coeftest)
importFrom(rlang,.data)
importFrom(sandwich,vcovHC)
importFrom(stats,AIC)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,df.residual)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,sd)
importFrom(stats,update)
importFrom(stats,vcov)

# ModLR

**Information-Theoretic Approach for Moderation Analysis**

## 📦 Overview

`ModLR` implements an information-theoretic framework for moderation analysis using multi-model inference based on Akaike's Information Criterion (AIC and AICc).

The package enables researchers to compare alternative moderation models and reduces the risk of identifying spurious moderation effects arising from nonlinear relationships.

---

## 📖 Citation

If you use this package, please cite:

> Daryanto, A. (2019). *Avoiding spurious moderation effects: An information-theoretic approach to moderation analysis*. Journal of Business Research, 103, 110–118.

This package provides an implementation of the information-theoretic framework proposed in this study.

## ✨ Key Features

- ✅ Multi-model comparison using AIC / AICc
- ✅ Detection of spurious moderation due to nonlinearity
- ✅ Supports continuous and categorical moderators
- ✅ Simple slopes analysis
- ✅ Johnson–Neyman intervals
- ✅ Visualization of moderation effects

---

## 📥 Installation

```r
# Install from CRAN (after acceptance)
install.packages("ModLR")

# Load package
library(ModLR)

#' Compare Moderation Models
#'
#' Compares candidate moderation models using information criteria (AIC/AICc).
#'
#' @param object A "modlr" object
#' @param models Optional numeric vector specifying which models to compare.
#'   If NULL (default), all candidate models are evaluated.
#' @param corrected Logical; whether to use AICc
#'
#' @return A data frame with model comparison results
#'
#' @examples
#' set.seed(123)
#'
#' n <- 100
#' x <- rnorm(n)
#' z <- 0.5 * x + sqrt(1 - 0.5^2) * rnorm(n)
#'
#' y <- 0.3 * x + 0.3 * z + 0.8 * x * z + rnorm(n)
#'
#' data <- data.frame(x, z, y)
#'
#' result <- moderated_regression(data, iv = "x", moderator = "z", dv = "y")
#'
#' compare_models(result)
#' compare_models(result, models = c(1, 2))
#'
#' @export

compare_models <- function(object, models = NULL, corrected = TRUE) {

  if (!inherits(object, "modlr"))
    stop("object must be of class modlr")

  data <- object$data
  dv <- as.character(object$call$dv)

  if (object$center) {
    x_var <- "x_center"
    z_var <- "z_center"
  } else {
    x_var <- object$iv
    z_var <- object$moderator
  }


  n <- nrow(data)

  # ✅ Available models
  all_models <- 1:8
  if (is.null(models)) models <- all_models

  if (!all(models %in% all_models)) {
    stop("Invalid model indices. Must be between 1 and 8.")
  }

  # ✅ IC decision
  k_max <- 8
  use_aicc <- corrected && ((n / k_max) < 40)
  ic_name <- ifelse(use_aicc, "AICc", "AIC")


  # Covariates (aligned with moderated_regression)

  covariates <- object$covariates

  if (is.null(covariates) || length(covariates) == 0) {
    cov_part <- NULL
  } else {
    if (object$center) {
      cov_names <- paste0(covariates, "_center")
    } else {
      cov_names <- covariates
    }
    cov_part <- paste(cov_names, collapse = " + ")
  }

  # Construct terms
  data$x_z   <- data[[x_var]] * data[[z_var]]
  data$x_sq  <- data[[x_var]]^2
  data$z_sq  <- data[[z_var]]^2
  data$z_xsq <- data[[z_var]] * data$x_sq
  data$x_zsq <- data[[x_var]] * data$z_sq

  # Base formula
  base_formula <- if (!is.null(cov_part)) {
    paste(dv, "~", cov_part)
  } else paste(dv, "~ 1")

  # Define once (used by multiple models)
  terms5 <- paste(base_formula, "+", x_var, "+", z_var)
  if (object$type_iv == "continuous") terms5 <- paste(terms5, "+ x_sq")
  if (object$type_moderator == "continuous") terms5 <- paste(terms5, "+ z_sq")

  # IC function
  calc_ic <- function(model) {
    k <- length(coef(model)) + 1
    aic <- AIC(model)
    if (use_aicc) {
      aic + (2 * k^2 + 2 * k) / (n - k - 1)
    } else {
      aic
    }
  }

  # Initialize
  model_names <- character(8)
  ic_vals <- params <- k_vals <- rep(NA, 8)

  # Model 1
  model_names[1] <- "1: x + z"
  if (1 %in% models) {
    m1 <- lm(as.formula(paste(base_formula, "+", x_var, "+", z_var)), data)
    ic_vals[1] <- calc_ic(m1)
    params[1] <- length(coef(m1))
    k_vals[1] <- params[1] + 1
  }

  # Model 2
  model_names[2] <- "2: x + x^2"
  if (2 %in% models && object$type_iv == "continuous") {
    m2 <- lm(as.formula(paste(base_formula, "+", x_var, "+ x_sq")), data)
    ic_vals[2] <- calc_ic(m2)
    params[2] <- length(coef(m2))
    k_vals[2] <- params[2] + 1
  }

  # Model 3
  model_names[3] <- "3: z + z^2"
  if (3 %in% models && object$type_moderator == "continuous") {
    m3 <- lm(as.formula(paste(base_formula, "+", z_var, "+ z_sq")), data)
    ic_vals[3] <- calc_ic(m3)
    params[3] <- length(coef(m3))
    k_vals[3] <- params[3] + 1
  }

  # Model 4
  model_names[4] <- "4: x + z + x*z"
  if (4 %in% models) {
    m4 <- lm(as.formula(paste(base_formula, "+", x_var, "+", z_var, "+ x_z")), data)
    ic_vals[4] <- calc_ic(m4)
    params[4] <- length(coef(m4))
    k_vals[4] <- params[4] + 1
  }

  # Model 5
  model_names[5] <- "5: x + z + x^2"
  if (5 %in% models) {
    m5 <- lm(as.formula(terms5), data)
    ic_vals[5] <- calc_ic(m5)
    params[5] <- length(coef(m5))
    k_vals[5] <- params[5] + 1
  }

  # Model 6
  model_names[6] <- "6: x + z + x*z + x^2"
  if (6 %in% models) {
    m6 <- lm(as.formula(paste(terms5, "+ x_z")), data)
    ic_vals[6] <- calc_ic(m6)
    params[6] <- length(coef(m6))
    k_vals[6] <- params[6] + 1
  }

  # Model 7
  model_names[7] <- "7: x + z + x*z + x^2 + z*x^2"
  if (7 %in% models && object$type_iv == "continuous") {
    m7 <- lm(as.formula(paste(
      base_formula, "+", x_var, "+", z_var,
      "+ x_z + x_sq + z_xsq"
    )), data)

    ic_vals[7] <- calc_ic(m7)
    params[7] <- length(coef(m7))
    k_vals[7] <- params[7] + 1
  }

  # Model 8
  model_names[8] <- "8: x + z + x*z + x^2 + x*z^2"
  if (8 %in% models && object$type_moderator == "continuous") {
    m8 <- lm(as.formula(paste(terms5, "+ x_z + x_zsq")), data)
    ic_vals[8] <- calc_ic(m8)
    params[8] <- length(coef(m8))
    k_vals[8] <- params[8] + 1
  }

  # Compute IC metrics
  valid <- which(!is.na(ic_vals))
  min_ic <- min(ic_vals[valid])

  delta <- likelihood <- weight <- rep(NA, 8)
  delta[valid] <- ic_vals[valid] - min_ic
  likelihood[valid] <- exp(-0.5 * delta[valid])
  weight[valid] <- likelihood[valid] / sum(likelihood[valid])

  rank_vals <- rep(NA, 8)
  rank_vals[valid] <- rank(ic_vals[valid], ties.method = "first")

  results <- data.frame(
    Model = model_names,
    Rank = rank_vals,
    Params = params,
    K = k_vals,
    IC = round(ic_vals, 2),
    Delta = round(delta, 2),
    Likelihood = round(likelihood, 3),
    Weight = round(weight, 3),
    Status = ifelse(is.na(ic_vals), "Skipped", "Estimated"),
    stringsAsFactors = FALSE
  )

  colnames(results)[colnames(results) == "IC"] <- ic_name
  colnames(results)[colnames(results) == "Delta"] <- paste0("Delta_", ic_name)

  width <- max(nchar(results$Model))
  results$Model <- format(results$Model, width = width, justify = "left")

  best_idx <- which.min(results[[ic_name]])
  best_row <- results[best_idx, ]

  message("\nBest model (by ", ic_name, "):")
  message(best_row$Model)
  message(ic_name, " = ", best_row[[ic_name]],
          " | Weight = ", round(best_row$Weight, 3), "\n")

  # -------------------------------
  # Covariate note
  # -------------------------------
  if (!is.null(object$covariates) && length(object$covariates) > 0) {
    message("Note: Results are conditional on the included covariates (held constant).\n")
  }

  return(results)


}

#' Simple Slopes Analysis
#'
#' Computes simple slopes for moderation analysis.
#'
#' @param object A fitted model
#' @param values Moderator values at which to compute slopes
#'
#' @return A data frame of slopes
#' @importFrom stats coef vcov lm df.residual pt pnorm qnorm qt
#' @importFrom sandwich vcovHC
#'
#' @examples
#' set.seed(123)
#'
#' n <- 100
#' x <- rnorm(n)
#' z <- 0.5 * x + sqrt(1 - 0.5^2) * rnorm(n)
#'
#' y <- 0.3 * x + 0.3 * z + 0.8 * x * z + rnorm(n)
#'
#' data <- data.frame(x, z, y)
#'
#' result <- moderated_regression(data, iv = "x", moderator = "z", dv = "y")
#'
#' simple_slopes(result)
#'
#' @export

simple_slopes <- function(object, values = NULL) {

  if (!inherits(object, "modlr")) stop("object must be of class modlr")

  data <- object$data
  type_mod <- object$type_moderator

  iv  <- object$iv
  mod <- object$moderator
  dv  <- object$dv


  # -------------------------------
  # Variable names (ALIGN WITH CENTERING)
  # -------------------------------
  if (object$center) {
    iv_name  <- paste0(iv, "_center")
    mod_name <- paste0(mod, "_center")
  } else {
    iv_name  <- iv
    mod_name <- mod
}

  # Use raw model

  mod_raw <- object$model


  b <- coef(mod_raw)

  # Variance structure
  if (object$robust_se) {
    vc <- sandwich::vcovHC(mod_raw, type = "HC3")
    crit <- qnorm(0.975)
  } else {
    vc <- vcov(mod_raw)
    crit <- qt(0.975, df.residual(mod_raw))
  }

  # SAFE coefficient extraction
  b1_name <- iv_name
  b3_name1 <- paste0(iv_name, ":", mod_name)
  b3_name2 <- paste0(mod_name, ":", iv_name)

  if (!b1_name %in% names(b)) stop("IV not found")

  if (b3_name1 %in% names(b)) {
    b3_name <- b3_name1
  } else if (b3_name2 %in% names(b)) {
    b3_name <- b3_name2
  } else if ("int_term" %in% names(b)) {
    b3_name <- "int_term"
  } else {
    stop("Interaction term not found")
  }

  b1 <- b[b1_name]
  b3 <- b[b3_name]

  # Variance components
  var_b1 <- vc[b1_name, b1_name]
  var_b3 <- vc[b3_name, b3_name]
  cov_b1b3 <- vc[b1_name, b3_name]

  # ===============================
  # BINARY MODERATOR
  # ===============================
  if (type_mod == "binary") {

    slope_0 <- b1
    slope_1 <- b1 + b3

    var_0 <- var_b1
    var_1 <- var_b1 + var_b3 + 2 * cov_b1b3

    se_0 <- sqrt(var_0)
    se_1 <- sqrt(var_1)

    t_0 <- slope_0 / se_0
    t_1 <- slope_1 / se_1

    if (object$robust_se) {
      p_0 <- 2 * (1 - pnorm(abs(t_0)))
      p_1 <- 2 * (1 - pnorm(abs(t_1)))
    } else {
      df_res <- df.residual(mod_raw)
      p_0 <- 2 * pt(abs(t_0), df_res, lower.tail = FALSE)
      p_1 <- 2 * pt(abs(t_1), df_res, lower.tail = FALSE)
    }

    results <- data.frame(
      Moderator = c("0", "1"),
      Slope = round(c(slope_0, slope_1), 3),
      SE = round(c(se_0, se_1), 3),
      t = round(c(t_0, t_1), 3),
      p_value = round(c(p_0, p_1), 4)
    )

  } else {

    # ===============================
    # CONTINUOUS MODERATOR
    # ===============================

    z <- data[[mod]]

    if (is.null(values)) {
      z_mean <- mean(z, na.rm = TRUE)
      z_sd   <- sd(z, na.rm = TRUE)
      values <- c(z_mean - z_sd, z_mean, z_mean + z_sd)
      labels <- c("M - 1 SD", "M (Mean)", "M + 1 SD")
    } else {
      labels <- paste("z =", round(values, 3))
    }

    results <- data.frame(
      Moderator_Level = labels,
      Moderator_Value = round(values, 3),
      Slope = NA, SE = NA, t = NA, p_value = NA
    )

    for (i in seq_along(values)) {

      z_val <- values[i]

      slope <- b1 + b3 * z_val

      var_slope <- var_b1 + z_val^2 * var_b3 + 2 * z_val * cov_b1b3
      se_slope <- sqrt(var_slope)

      t_val <- slope / se_slope

      if (object$robust_se) {
        p_val <- 2 * (1 - pnorm(abs(t_val)))
      } else {
        p_val <- 2 * pt(abs(t_val), df.residual(mod_raw), lower.tail = FALSE)
      }

      results$Slope[i]   <- round(slope, 3)
      results$SE[i]      <- round(se_slope, 3)
      results$t[i]       <- round(t_val, 3)
      results$p_value[i] <- round(p_val, 4)
    }
  }

  message("\n========================================\n")
  message("SIMPLE SLOPES ANALYSIS\n")
  message("========================================\n")

  if (object$robust_se) {
    message("(Using HC3 robust standard errors)\n")
  }

  # -------------------------------
  # Covariate note
  # -------------------------------
  if (!is.null(object$covariates) && length(object$covariates) > 0) {
    message("Note: Results are conditional on the included covariates (held constant).\n")
  }

  return(results)


}

#' Plot Moderation Effect
#'
#' Produces a plot of the moderation effect.
#'
#' @param object A fitted model
#'
#' @return A ggplot object
#'
#' @examples
#' set.seed(123)
#'
#' n <- 100
#' x <- rnorm(n)
#' z <- 0.5 * x + sqrt(1 - 0.5^2) * rnorm(n)
#'
#' y <- 0.3 * x + 0.3 * z + 0.8 * x * z + rnorm(n)
#'
#' data <- data.frame(x, z, y)
#'
#' result <- moderated_regression(data, iv = "x", moderator = "z", dv = "y")
#'
#' plot_moderation(result)
#'
#' @export

plot_moderation <- function(object) {

  if (!inherits(object, "modlr")) {
    stop("object must be of class modlr")
  }

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("ggplot2 package required")
  }

  data <- object$data
  iv   <- object$iv
  mod  <- object$moderator
  dv   <- object$dv

  # ✅ Use RAW model (consistent with JN + slopes)
  mod_raw <- lm(
    as.formula(paste(dv, "~", iv, "*", mod)),
    data = data
  )

  # ✅ Generate x sequence
  x_seq <- seq(
    min(data[[iv]], na.rm = TRUE),
    max(data[[iv]], na.rm = TRUE),
    length.out = 100
  )

  # ===============================
  # ✅ Define moderator values
  # ===============================
  if (object$type_moderator == "binary") {

    z_vals <- sort(unique(data[[mod]]))
    labels <- paste(mod, "=", z_vals)

  } else {

    z_mean <- mean(data[[mod]], na.rm = TRUE)
    z_sd   <- sd(data[[mod]], na.rm = TRUE)

    z_vals <- c(z_mean - z_sd, z_mean, z_mean + z_sd)
    labels <- c("Low (-1 SD)", "Mean", "High (+1 SD)")
  }

  # ===============================
  # ✅ Build prediction data
  # ===============================
  pred_data <- data.frame()

  for (i in seq_along(z_vals)) {

    temp <- data.frame(x_seq, rep(z_vals[i], length(x_seq)))
    names(temp) <- c(iv, mod)

    # ✅ Predict from model (handles covariates automatically)
    temp$.pred <- predict(mod_raw, newdata = temp)

    temp$group <- labels[i]

    pred_data <- rbind(pred_data, temp)
  }

  # ===============================
  # ✅ Plot
  # ===============================
  p <- ggplot2::ggplot(
    pred_data,
    ggplot2::aes(x = .data[[iv]], y = .data[[".pred"]], color = .data[["group"]])
    ) +
    ggplot2::geom_line(linewidth = 1.2) +
    ggplot2::labs(
      title = "Moderation Effect",
      subtitle = paste("Interaction p =", round(object$interaction_p, 4)),
      x = iv,
      y = paste("Predicted", dv),
      color = "Moderator"
    ) +
    ggplot2::theme_bw() +
    ggplot2::theme(legend.position = "bottom")


  # -------------------------------
  # Covariate note
  # -------------------------------
  if (!is.null(object$covariates) && length(object$covariates) > 0) {
    message("Note: Results are conditional on the included covariates (held constant).\n")
  }

  return(p)


}

#' @export
#' @method print modlr
print.modlr <- function(x, ...) {

  message("\nModerated Regression Model\n")
  message("==========================\n\n")

  print(summary(x$model))

  if (!is.null(x$interaction_p)) {
    pval <- x$interaction_p

    pval_str <- if (is.na(pval)) {
      "NA"
    } else if (pval < 0.001) {
      "< 0.001"
    } else {
      format(round(pval, 4), nsmall = 4)
    }

    message("\n---------------------------------\n")
    message("Interaction p-value:", pval_str)

    if (!is.na(pval) && pval < 0.05) {
      message(" (Significant)\n")
    } else {
      message(" (Not significant)\n")
    }
    message("\n")
  }
}

#' ModLR package
#'
#' @importFrom broom tidy
#' @importFrom ggplot2 ggplot aes geom_point geom_line labs theme
#' @importFrom stats AIC as.formula coef complete.cases df.residual lm na.omit predict pt qt sd vcov
#' @importFrom rlang .data
#' @keywords internal
"_PACKAGE"

#' ModLR: Information-Theoretic Moderation Analysis
#'
#' The ModLR package implements an information-theoretic framework for moderation analysis
#' using multi-model inference based on Akaike's Information Criterion (AIC and AICc).
#'
#' It enables researchers to compare alternative moderation models and reduces the risk
#' of spurious moderation effects arising from nonlinear relationships.
#'
#' The methodology is based on Daryanto (2019), which introduces an
#' information-theoretic approach to moderation analysis.
#'
#' @references
#' Daryanto, A. (2019). Avoiding spurious moderation effects:
#' An information-theoretic approach to moderation analysis.
#' Journal of Business Research, 103, 110–118.
#'
#' @keywords internal
"_PACKAGE"

#' Johnson-Neyman Analysis
#'
#' Computes regions of significance for an interaction effect.
#'
#' @param object A fitted model (modlr object)
#' @param alpha Significance level
#' @param robust Logical; use HC3 robust standard errors
#'
#' @return A data.frame of Johnson-Neyman results
#'
#' @importFrom stats qt qnorm coef vcov df.residual
#' @importFrom sandwich vcovHC
#'
#' @examples
#' set.seed(123)
#'
#' n <- 100
#' x <- rnorm(n)
#' z <- 0.5 * x + sqrt(1 - 0.5^2) * rnorm(n)
#' y <- 0.3 * x + 0.3 * z + 0.8 * x * z + rnorm(n)
#'
#' data <- data.frame(x, z, y)
#'
#' result <- moderated_regression(data, iv = "x", moderator = "z", dv = "y")
#'
#' johnson_neyman(result)
#' johnson_neyman(result, robust = TRUE)
#'
#' @export

johnson_neyman <- function(object, alpha = 0.05, robust = NULL) {

  if (!inherits(object, "modlr")) stop("object must be of class modlr")

  if (object$type_moderator != "continuous") {
    stop("Johnson-Neyman technique is only for continuous moderators")
  }

  data <- object$data

  # -------------------------------
  # Variable names (ALIGN WITH CENTERING)
  # -------------------------------
  if (object$center) {
    x_var <- "x_center"
    z_var <- "z_center"
  } else {
    x_var <- object$iv
    z_var <- object$moderator
  }

  if (is.null(robust)) {
    robust <- object$robust_se
  }

  # ✅ Use original model
  model <- object$model
  b <- coef(model)

  vc <- if (robust) {
    sandwich::vcovHC(model, type = "HC3")
  } else {
    vcov(model)
  }

  # ✅ Centering constant from model (exact)
  zc <- data[[z_var]]
  z  <- data[[object$moderator]]

  z_mean <- mean(z - zc, na.rm = TRUE)

  # ✅ Recover RAW-scale coefficients (PROCESS logic)

  b1 <- b[x_var] - b["int_term"] * z_mean


  b3 <- b["int_term"]



  # Transform variance-covariance matrix

  var_xc  <- vc[x_var, x_var]
  var_int <- vc["int_term", "int_term"]
  cov_xc_int <- vc[x_var, "int_term"]


  var_b1 <- var_xc + z_mean^2 * var_int - 2 * z_mean * cov_xc_int
  var_b3 <- var_int
  cov_b1b3 <- cov_xc_int - z_mean * var_int

  # ✅ Critical value

   tcrit <- qt(1 - alpha / 2, df = df.residual(model))


  # ✅ ANALYTIC J–N SOLUTION (raw scale)
  A <- b3^2 - tcrit^2 * var_b3
  B <- 2 * b1 * b3 - 2 * tcrit^2 * cov_b1b3
  C <- b1^2 - tcrit^2 * var_b1

  disc <- B^2 - 4 * A * C

  if (disc < 0) {
    message("\nNo Johnson-Neyman regions found.\n\n")
    return(NULL)
  }

  z1_raw <- (-B - sqrt(disc)) / (2 * A)
  z2_raw <- (-B + sqrt(disc)) / (2 * A)

  z_original <- sort(c(z1_raw, z2_raw))

  # ✅ Convert back to CENTERED scale for your output
  z_centered <- z_original - z_mean

  results <- data.frame(
    Region = c("Lower", "Upper"),
    z_centered = round(z_centered, 3),
    z_original = round(z_original, 4)
  )

  # -------------------------------
  # Covariate note
  # -------------------------------
  if (!is.null(object$covariates) && length(object$covariates) > 0) {
    message("Note: Results are conditional on the included covariates (held constant).\n")
  }

    return(results)


}

#' Moderated Regression Model
#'
#' Fits a moderated regression model with optional extensions.
#'
#' @param data A data frame
#' @param dv Dependent variable
#' @param iv Independent variable
#' @param moderator Moderator variable
#' @param covariates Optional character vector of covariate names.
#'   Defaults to NULL (no covariates).
#' @param center Logical; whether to center variables
#' @param quadratic Logical; include quadratic terms
#' @param robust_se Logical; use HC3 robust standard errors
#'
#' @return A fitted model object
#' @importFrom stats lm coef vcov update
#' @importFrom sandwich vcovHC
#' @importFrom lmtest coeftest
#'
#' @examples
#' set.seed(123)
#'
#' n <- 100
#' x <- rnorm(n)
#' w1 <- rnorm(n)
#' w2 <- rnorm(n)
#' z <- 0.5 * x + sqrt(1 - 0.5^2) * rnorm(n)
#'
#' y <- 0.3 * x + 0.3 * z + 0.8 * x * z + rnorm(n)
#'
#' data <- data.frame(w1, w2, x, z, y)
#'
#'result <- moderated_regression(
#'  data,
#'  iv = "x",
#'  moderator = "z",
#'  dv = "y",
#'  covariates = c("w1", "w2")
#'  )
#'
#'
#' @export
#'
moderated_regression <- function(data, iv, moderator, dv,
                                 covariates = NULL,
                                 center = TRUE,
                                 quadratic = FALSE,
                                 robust_se = FALSE) {

  detect_type <- function(x) {
    if (is.factor(x)) return("binary")
    unique_vals <- unique(na.omit(x))
    if (length(unique_vals) <= 2) return("binary")
    return("continuous")
  }

  # -------------------------------
  # Validate input
  # -------------------------------
  if (!is.data.frame(data)) stop("data must be a data frame")

  if (is.null(covariates)) covariates <- character(0)

  required_vars <- c(iv, moderator, dv, covariates)

  missing_vars <- required_vars[!required_vars %in% names(data)]
  if (length(missing_vars) > 0) {
    stop("Missing variables: ", paste(missing_vars, collapse = ", "))
  }

  data_clean <- data[complete.cases(data[, required_vars, drop = FALSE]), ]
  if (nrow(data_clean) < 10) stop("Insufficient observations")

  type_iv <- detect_type(data_clean[[iv]])
  type_moderator <- detect_type(data_clean[[moderator]])

  df <- data_clean

  # -------------------------------
  # Centering (UPDATED NAMING)
  # -------------------------------
  if (center) {
    x_var <- "x_center"
    z_var <- "z_center"

    df[[x_var]] <- scale(df[[iv]], center = TRUE, scale = FALSE)
    df[[z_var]] <- scale(df[[moderator]], center = TRUE, scale = FALSE)
  } else {
    x_var <- iv
    z_var <- moderator

    df[[x_var]] <- df[[iv]]
    df[[z_var]] <- df[[moderator]]
  }

  # -------------------------------
  # Interaction
  # -------------------------------
  df$int_term <- df[[x_var]] * df[[z_var]]

  # -------------------------------
  # Covariates (UPDATED NAMING)
  # -------------------------------
  if (length(covariates) > 0) {

    cov_names <- c()

    for (cov in covariates) {

      if (center && is.numeric(df[[cov]])) {
        new_name <- paste0(cov, "_center")
        df[[new_name]] <- scale(df[[cov]], center = TRUE, scale = FALSE)
      } else {
        new_name <- cov
        df[[new_name]] <- df[[cov]]
      }

      cov_names <- c(cov_names, new_name)
    }

    cov_terms <- paste(cov_names, collapse = " + ")

    formula_base <- as.formula(
      paste(dv, "~", cov_terms, "+", x_var, "+", z_var, "+ int_term")
    )

  } else {

    formula_base <- as.formula(
      paste(dv, "~", x_var, "+", z_var, "+ int_term")
    )
  }

  # -------------------------------
  # Quadratic terms (UPDATED NAMING)
  # -------------------------------
  if (quadratic &&
      type_iv == "continuous" &&
      type_moderator == "continuous") {

    x_sq <- paste0(x_var, "_sq")
    z_sq <- paste0(z_var, "_sq")

    df[[x_sq]] <- df[[x_var]]^2
    df[[z_sq]] <- df[[z_var]]^2

    formula_final <- update(formula_base,
                            as.formula(paste(". ~ . +", x_sq, "+", z_sq)))
  } else {
    formula_final <- formula_base
  }

  # -------------------------------
  # Model estimation
  # -------------------------------
  model <- lm(formula_final, data = df)

  coef_summary <- summary(model)$coefficients

  interaction_p <- if ("int_term" %in% rownames(coef_summary)) {
    coef_summary["int_term", "Pr(>|t|)"]
  } else {
    NA_real_
  }

  # -------------------------------
  # Robust SE
  # -------------------------------
  if (robust_se) {
    vc <- sandwich::vcovHC(model, type = "HC3")
    coef_table <- lmtest::coeftest(model, vc)
  } else {
    vc <- vcov(model)
    coef_table <- coef_summary
  }

  # -------------------------------
  # Output
  # -------------------------------
  result <- list(
    model = model,
    vcov = vc,
    coef_table = coef_table,
    interaction_p = interaction_p,
    type_iv = type_iv,
    type_moderator = type_moderator,
    covariates = covariates,
    center = center,
    quadratic = quadratic,
    robust_se = robust_se,
    iv = iv,
    moderator = moderator,
    dv = dv,
    data = df,
    call = match.call()
  )

  class(result) <- "modlr"
  return(result)
}

---
title: "Introduction to ModLR"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to ModLR}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r setup, include=FALSE}
library(ModLR)
```

# Overview
ModLR provides an information-theoretic framework for moderation analysis
using multi-model inference based on Akaike's Information Criterion (AIC and AICc).
Traditional approaches to moderation analysis typically rely on a single model
with an interaction term. However, such approaches may lead to spurious
moderation effects when nonlinear relationships are present but not properly modeled.
The ModLR package addresses this limitation by comparing multiple candidate
models and identifying the most appropriate specification based on information
criteria.

# Basic Example

```{r, eval=FALSE}

set.seed(123)

n <- 400

z <- sample(c(0, 1), n, replace = TRUE)
x <- ifelse(z == 0, runif(n, 0.5, 6.5), runif(n, 3.5, 9.5))
y <- (0.8 * x^2) + (30 * z) + rnorm(n, 0, 5)

ddat <- data.frame(x = x, y = y, z = z)

result <- moderated_regression(dat, iv = "x", moderator = "z", dv = "y")
print(result)
compare_models(result)
```

The output shows competing models ranked according to AIC or AICc. Lower values
indicate better model fit, allowing the researcher to identify whether the
interaction model is truly supported by the data.

# Interpreting Model Comparison

The ModLR framework evaluates alternative model specifications rather than relying
on a single model. This approach helps determine:

- whether the interaction model improves fit
- whether nonlinear models provide a better explanation
- the relative strength of evidence for each model

By focusing on model comparison, researchers can avoid incorrectly concluding that
moderation is present when it is actually due to model misspecification.

# Additional Analyses

## Simple Slopes

Simple slopes estimate the effect of the independent variable at specific levels
of the moderator, helping clarify how the relationship changes across conditions.


```{r, eval=FALSE}
simple_slopes(result)
```


## Johnson–Neyman Analysis

The Johnson–Neyman procedure identifies the range of moderator values for which
the effect of the independent variable is statistically significant.


```{r, eval=FALSE}
johnson_neyman(result)
```

## Visualization

Visualization helps communicate moderation effects clearly by showing how the
relationship between variables changes across levels of the moderator.


```{r, eval=FALSE}
plot_moderation(result)
```

# Methodological Background

The framework implemented in ModLR is based on Daryanto (2019), which proposes
an information-theoretic approach to moderation analysis. By comparing alternative
models using AIC/AICc, this approach reduces the risk of identifying spurious
moderation effects arising from nonlinear relationships.

# Conclusion

ModLR provides a principled approach to moderation analysis by integrating
multi-model inference with interpretation tools. The package allows researchers
to go beyond traditional single-model approaches and to conduct more robust and
reliable analyses.

# References

Daryanto, A. (2019). Avoiding spurious moderation effects: An information-theoretic
approach to moderation analysis. Journal of Business Research, 103, 110–118.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_moderation.R
\name{plot_moderation}
\alias{plot_moderation}
\title{Plot Moderation Effect}
\usage{
plot_moderation(object)
}
\arguments{
\item{object}{A fitted model}
}
\value{
A ggplot object
}
\description{
Produces a plot of the moderation effect.
}
\examples{
set.seed(123)

n <- 100
x <- rnorm(n)
z <- 0.5 * x + sqrt(1 - 0.5^2) * rnorm(n)

y <- 0.3 * x + 0.3 * z + 0.8 * x * z + rnorm(n)

data <- data.frame(x, z, y)

result <- moderated_regression(data, iv = "x", moderator = "z", dv = "y")

plot_moderation(result)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_models.R
\name{compare_models}
\alias{compare_models}
\title{Compare Moderation Models}
\usage{
compare_models(object, models = NULL, corrected = TRUE)
}
\arguments{
\item{object}{A "modlr" object}

\item{models}{Optional numeric vector specifying which models to compare.
If NULL (default), all candidate models are evaluated.}

\item{corrected}{Logical; whether to use AICc}
}
\value{
A data frame with model comparison results
}
\description{
Compares candidate moderation models using information criteria (AIC/AICc).
}
\examples{
set.seed(123)

n <- 100
x <- rnorm(n)
z <- 0.5 * x + sqrt(1 - 0.5^2) * rnorm(n)

y <- 0.3 * x + 0.3 * z + 0.8 * x * z + rnorm(n)

data <- data.frame(x, z, y)

result <- moderated_regression(data, iv = "x", moderator = "z", dv = "y")

compare_models(result)
compare_models(result, models = c(1, 2))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ModLR-package.R, R/imports.R
\docType{package}
\name{ModLR-package}
\alias{ModLR-package}
\title{ModLR: Information-Theoretic Moderation Analysis}
\description{
The ModLR package implements an information-theoretic framework for moderation analysis
using multi-model inference based on Akaike's Information Criterion (AIC and AICc).

Provides a robust implementation of information-theoretic moderation analysis using multi-model inference based on Akaike's Information Criterion (AIC) and its small-sample corrected form (Corrected AIC). The package enables researchers to compare competing model specifications and helps distinguish true interaction effects from nonlinear relationships that may produce spurious moderation. The methods build on Daryanto (2019) \doi{10.1016/j.jbusres.2019.06.012}.
}
\details{
It enables researchers to compare alternative moderation models and reduces the risk
of spurious moderation effects arising from nonlinear relationships.

The methodology is based on Daryanto (2019), which introduces an
information-theoretic approach to moderation analysis.
}
\references{
Daryanto, A. (2019). Avoiding spurious moderation effects:
An information-theoretic approach to moderation analysis.
Journal of Business Research, 103, 110–118.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/ahdar1/ModLR}
}

}
\author{
\strong{Maintainer}: Ahmad Daryanto \email{ahdar_2000@yahoo.com}

Authors:
\itemize{
  \item Ahmad Daryanto \email{ahdar_2000@yahoo.com}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/moderated_regression.R
\name{moderated_regression}
\alias{moderated_regression}
\title{Moderated Regression Model}
\usage{
moderated_regression(
  data,
  iv,
  moderator,
  dv,
  covariates = NULL,
  center = TRUE,
  quadratic = FALSE,
  robust_se = FALSE
)
}
\arguments{
\item{data}{A data frame}

\item{iv}{Independent variable}

\item{moderator}{Moderator variable}

\item{dv}{Dependent variable}

\item{covariates}{Optional character vector of covariate names.
Defaults to NULL (no covariates).}

\item{center}{Logical; whether to center variables}

\item{quadratic}{Logical; include quadratic terms}

\item{robust_se}{Logical; use HC3 robust standard errors}
}
\value{
A fitted model object
}
\description{
Fits a moderated regression model with optional extensions.
}
\examples{
set.seed(123)

n <- 100
x <- rnorm(n)
w1 <- rnorm(n)
w2 <- rnorm(n)
z <- 0.5 * x + sqrt(1 - 0.5^2) * rnorm(n)

y <- 0.3 * x + 0.3 * z + 0.8 * x * z + rnorm(n)

data <- data.frame(w1, w2, x, z, y)

result <- moderated_regression(
 data,
 iv = "x",
 moderator = "z",
 dv = "y",
 covariates = c("w1", "w2")
 )


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/johnson_neyman.R
\name{johnson_neyman}
\alias{johnson_neyman}
\title{Johnson-Neyman Analysis}
\usage{
johnson_neyman(object, alpha = 0.05, robust = NULL)
}
\arguments{
\item{object}{A fitted model (modlr object)}

\item{alpha}{Significance level}

\item{robust}{Logical; use HC3 robust standard errors}
}
\value{
A data.frame of Johnson-Neyman results
}
\description{
Computes regions of significance for an interaction effect.
}
\examples{
set.seed(123)

n <- 100
x <- rnorm(n)
z <- 0.5 * x + sqrt(1 - 0.5^2) * rnorm(n)
y <- 0.3 * x + 0.3 * z + 0.8 * x * z + rnorm(n)

data <- data.frame(x, z, y)

result <- moderated_regression(data, iv = "x", moderator = "z", dv = "y")

johnson_neyman(result)
johnson_neyman(result, robust = TRUE)

}
